/**
 * @fileoverview Do not hardcode content. Use appropriate i18n keys
 * @author Aravindh T
 */
'use strict';//no i18n

const {RuleTester} = require('eslint')
var rule = require('../../../lib/rules/no-hardcoding')


RuleTester.setDefaultConfig({
    env :{
        browser : true,
        commonjs: true,
        es6: true,
        node: true
      },
      parserOptions: {
        ecmaVersion: 12,
        ecmaFeatures: {
          jsx: true
        },
        sourceType: 'module'
      }
})
const options = [{
    objectKeys:["dataId","data-id","data-test-id","size","iconSize","className"],
    props:["dataId","data-id","data-test-id","size","iconSize","className"],
    methods:["findIdex","findIndex","includes"],
    allowedTags:["svg"],
    keys:["Product Category","Product Owner","Product Information","Summarize information"],
    commentsToIgnoreHardcodingCheck:["no i18n"],
    stringsAllowedAsBlockCommentStart: ["ignore i18n start","no i18n start"],
    stringsAllowedAsBlockCommentEnd: ["ignore i18n end","no i18n end"],
    dateFormatPatterns:["DD MMM YYYY hh:mm A","DD\/MM\/YYYY h:mm A","DDD, DD MMM YYYY HH:mm:ss","DD MMM YYYY","hh:mm A"],
    startString:["ZD-","zd-","GC-","https","..","ht"],
    allowedPropStartingWords:["is","has","need","animation"],
    allowedPropEndingWords:["Id","id","size","palette"],
    allowedWordsThatCanBePresentInAComponentNameToExcludeTheComponent:["Svg"]
}]

var ruleTester = new RuleTester();

ruleTester.run('no-hardcoding',rule,{
    valid : [
        {
            name: 'case1',
            code: `<Container className="test string" key='correct.format.key' method={someMethod('single')}/>`,
            options:options
        },
        {
            name: 'case2',
            code: 'dataId: string',
            options:options
        },
        {
            name: 'case3',
            code: `<Container text='hardcoded string' /> //no i18n`,
            options:options
        },
        {
            name: 'case4',
            code: `let index = findIndex('firstElement')`,
            options:options
        },
        {
            name: 'case5',
            code: `<svg title='incorrect hardcoded string' d='test'>
                        <g opacity="0.5">
                            <path test={'test string'} test1={'string '} xmlns="dttp://www.w3.org/2000/svg" dept="M61.468 14.7866C61.468 14.7866 62.1294 16.2436 63.6311 18.6654L65.3309 16.9784C62.9186 15.4576 61.468 14.7866 61.468 14.7866Z" fill="var(--zdt_tips_svg_primary_stroke)"/>
                            <rect a="0.0352379" b="0.0355971" widdth={'9 0 9 0'} heiight={'9.23 9.24'} rxa="0.67294" transformer="matrix(0.999987 0.00507125 -0.00506809 0.999987 48.7931 17.7964)" filll="#F0F3F7" stroke="#E5E5EE" strooke-width="0.0708358"/>
                        </g>
                    </svg>
            `,
            options:options
        },
        {
            name: 'case7',
            code: `
                let arr = [
                    {
                        displayLabel: 'hardcoded string', //ignore i18n start
                        dataId: 'string'
                    },
                    {
                        displayLabel: 'hardcoded string',
                        dataId: 'string'
                    },
                    {
                        displayLabel: 'hardcoded string',
                        dataId: 'string'
                    },
                    {
                        displayLabel: 'hardcoded string',
                        dataId: 'string'
                    },
                    {
                        displayLabel: 'hardcoded string', //ignore i18n end
                        dataId: 'string'
                    }
                ]
            `,
            options:options
        },
        {
            name: 'case6',
            code: `let i18nedValue = getI18NValue('Product Information')`,
            options:options
        },
        
        {
            name: 'case8',
            code: `<Container text='PascalString' />`,
            options:options
        },
        {
            name: 'case9',
            code: `let seqOfNums = "0 0 12 20" `,
            options:options
        },
        {
            name: 'case10',
            code: `
                let variable = 'someString'
                let test = variable === 'var' ? 'string one' : 'stringtwo'
            `,
            options:options
        },
        {
            name: 'case11',
            code: `
                const obj = {
                    dataId: 'hardcoded string',
                    iconSize: 'large',
                    'key':'value',
                    dynamic: 'string_hardcoded',
                    class: 'ZD-Icon',
                    empty: ' ',
                    symbol: '#'
                }
            `,
            options:options
        },
        {
            name: 'case12',
            code: `let num = obj['number']`,
            options:options
        },
        {
            name: 'case13',
            code: `<Container text='camelCase' number='100' bool='true' />`,
            options:options
        },
        {
            name: 'case14',
            code: `let dateFormat = 'DD MMM YYYY hh:mm A'`,
            options:options
        },
        {
            name: 'case15',
            code: `let cssProp = '10px #34rfe3'`,
            options:options
        },
        {
            name: 'case16',
            code: `throw 'hardstr'`,
            options:options
        },
        {
            name: 'case17',
            code: `
                function doSomething(){
                    return new Error('string')
                }
            `,
            options:options
        },
        {
            name: 'case18',
            code: `let someRegex = /^[A-Z]/`,
            options:options
        },
        {
            name: 'case19',
            code: `<Container>100</Container>`,
            options:options
        },
        {
            name: 'case20',
            code: `let test = 'singlewordhardcoding'`,
            options:options    
        }   
    ],
    invalid : [
        {
            code: `let test = "test variable"`,
            errors: [
                {
                message: 'Do not hardcode content. Use I18N key instead'//no i18n
                }
            ],
            options:options            
        },
        {

            code: `<Container displayLabel='hardcoded string' />`,
            errors:[
                {
                    message: 'Do not hardcode content. Use I18N key instead'//no i18n
                }
             ],
            options:options
        },
        {
            code:`
                let obj = {
                    key: 'hardcoded string'
                }
            `,            
            errors:[            
                {            
                    message: 'Do not hardcode content. Use I18N key instead'//no i18n            
                }            
             ],
            options:options
        },
        {
            code:`let test = 'singlewordhardcoding'
            const App = () => (
                <Container displayLabel={test}></Container>
            );`,
            errors:[
                {
                    message: 'The variable "test" is assigned a string and passed as prop value to the prop "displayLabel" in line 3 which may cause I18N issue. So use appropriate I18N key instead of assigning a string'
                }
             ],
            options:options
        },
        {
            code: `<Container displayLabel='Text' />`,
            errors:[
                {
                    message: 'Do not hardcode content. Use I18N key instead'
                }
             ],
            options:options

        },
        {
            code: `<Container>Text</Container>`,
            errors:[
                {
                    message: 'Do not hardcode string as JSX children in the JSX tag. Use i18n key instead'
                }
            ],
            options:options
        }
    ]
}
);
// console.log("All tests passed!")