/**
 * @fileoverview Use the getI18Nvalue method appropriately
 * @author Aravindh T
 */
'use strict';//no i18n

const { RuleTester } = require('eslint')
var rule = require('../../../lib/rules/use-getI18NValue-method-appropriately')


RuleTester.setDefaultConfig({
    env :{
        browser : true,
        commonjs: true,
        es6: true,
        node: true
      },
      parserOptions: {
        ecmaVersion: 6,
        ecmaFeatures: {
          jsx: true
        },
        sourceType: 'module'
      }
})

const options = [{
    i18nKeys: ['Product Owner','Product Information'],
    commentsToIgnoreCheck: ['no i18n']
}]

var ruleTester = new RuleTester();

ruleTester.run('no-hardcoding-in-getI18NValue-method',rule,{
    valid : [
        {
            code: `let a = getI18NValue('use.appropriate.key')`,
            options:options
        },
        {
            code: `let d = getI18NValue('incorrect usgae') //no i18n`,
            options:options
        }
    ],
    invalid: [
        {
            code: `let b = getI18NValue('sample key')`,
            errors: [
                {
                    message: 'Do not hardcode content inside getI18NValue method. Use I18N keys instead'//no i18n
                }                
            ],
            options:options
        },
        {
            code: `let c = getI18NValue('sample.key',getI18NValue('second.key'))`,
            errors:[
                {
                    message: 'Do not pass getI18NValue or its alias name as parameter to getI18NValue method'//no i18n
                }
            ],
            options:options
        },
        {
            code: `property: getI18NValue('sample.key') + getI18NValue('second.sample.key')`,
            errors:[
                {
                    message: 'Do not concat getI18NValue method. Use a single key'//no i18n
                }
            ],
            options:options
        }
    ]
}
)
console.log('All tests passed')